﻿using System;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows.Threading;
using Telerik.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.Docking;
using Telerik.Windows.Controls.GridView;
using Telerik.Windows.Controls.RichTextBoxUI;

namespace OutlookTemplate
{
    public class MainViewModel : ViewModelBase
    {
        private ObservableCollection<OutlookSection> outlookSections;
        private SelectedEmailViewModel selectedEmailViewModel;

        private Email selectedEmail;

        public MainViewModel()
        {
            this.OutlookSections = EmailService.GetOutlookSections();
            this.MarkUnreadCommand = new DelegateCommand(this.OnMarkUnreadCommandExecuted, o => this.SelectedEmail != null);
            this.MarkUnreadReadCommand = new DelegateCommand(this.OnMarkUnreadReadCommandExecuted, o => this.SelectedEmail != null);
            this.OpenedCommand = new DelegateCommand(this.OnOpenedCommandExecuted);
            this.MenuOpenStateChangedCommand = new DelegateCommand(this.OnMenuOpenStateChangedCommandExecuted);
            this.PreviewShowCompassCommand = new DelegateCommand(this.OnPreviewShowCompassCommandExecuted);
            this.OpenDialogCommand = new DelegateCommand(this.OnOpenDialogCommandExecuted);
            this.SelectedEmailViewModel = new SelectedEmailViewModel();
        }

        public DelegateCommand MarkUnreadCommand { get; private set; }
        public DelegateCommand MarkUnreadReadCommand { get; private set; }
        public DelegateCommand OpenedCommand { get; private set; }
        public DelegateCommand MenuOpenStateChangedCommand { get; private set; }
        public DelegateCommand PreviewShowCompassCommand { get; private set; }
        public DelegateCommand OpenDialogCommand { get; private set; }

        /// <summary>
        /// Gets or sets SelectedEmailViewModel and notifies for changes
        /// </summary>
		public SelectedEmailViewModel SelectedEmailViewModel
        {
            get
            {
                return this.selectedEmailViewModel;
            }
            set
            {
                if (this.selectedEmailViewModel != value)
                {
                    this.selectedEmailViewModel = value;
                    this.OnPropertyChanged(() => this.SelectedEmailViewModel);
                }
            }
        }

        /// <summary>
        /// Gets or sets OutlookSections and notifies for changes
        /// </summary>
        public ObservableCollection<OutlookSection> OutlookSections
        {
            get
            {
                return this.outlookSections;
            }

            set
            {
                if (this.outlookSections != value)
                {
                    this.outlookSections = value;
                    this.OnPropertyChanged(() => this.OutlookSections);
                }
            }
        }

        /// <summary>
        /// Gets or sets SelectedEmail and notifies for changes
        /// </summary>
        public Email SelectedEmail
        {
            get
            {
                return this.selectedEmail;
            }

            set
            {
                if (this.selectedEmail != value)
                {
                    this.selectedEmail = value;
                    this.SelectedEmailViewModel.Email = value;
                    this.MarkUnreadReadCommand.InvalidateCanExecute();
                    this.OnPropertyChanged(() => this.SelectedEmail);
                }
            }
        }

        private static bool CompassNeedsToShow(Compass compass)
        {
            return compass.IsLeftIndicatorVisible
                || compass.IsTopIndicatorVisible
                || compass.IsRightIndicatorVisible
                || compass.IsBottomIndicatorVisible
                || compass.IsCenterIndicatorVisible;
        }

        private void OnMarkUnreadCommandExecuted(object param)
        {
            this.SelectedEmail.Status = Enums.EmailStatus.Unread;
        }

        private void OnMarkUnreadReadCommandExecuted(object param)
        {
            this.SelectedEmail.Status = this.SelectedEmail.Status == Enums.EmailStatus.Unread ? Enums.EmailStatus.Read : Enums.EmailStatus.Unread;
        }

        private void OnOpenedCommandExecuted(object param)
        {
            var args = (RadRoutedEventArgs)param;
            var menu = (RadContextMenu)args.OriginalSource;
            var row = menu.GetClickedElement<GridViewRow>();
            if (row != null)
            {
                row.IsSelected = row.IsCurrent = true;
                GridViewCell cell = menu.GetClickedElement<GridViewCell>();
                if (cell != null)
                {
                    cell.IsCurrent = true;
                }
            }
            else
            {
                menu.IsOpen = false;
            }
        }

        private void OnMenuOpenStateChangedCommandExecuted(object param)
        {
            var ribbon = param as RadRichTextBoxRibbonUI;
            if (ribbon.IsApplicationMenuOpen)
            {
                if (this.SelectedEmailViewModel != null && this.SelectedEmailViewModel.DiscardCommand != null)
                {
                    Dispatcher.CurrentDispatcher.BeginInvoke(new Action(() =>
                    {
                        this.SelectedEmailViewModel.DiscardCommand.Execute(null);
                    }));
                }
            }
        }

        private void OnPreviewShowCompassCommandExecuted(object param)
        {
            var args = (PreviewShowCompassEventArgs)param;
            if (args.TargetGroup != null)
            {
                args.Compass.IsCenterIndicatorVisible = false;
                args.Compass.IsLeftIndicatorVisible = false;
                args.Compass.IsTopIndicatorVisible = false;
                args.Compass.IsRightIndicatorVisible = false;
                args.Compass.IsBottomIndicatorVisible = false;
            }
            else
            {
                args.Compass.IsCenterIndicatorVisible = CanDock(args.DraggedElement, DockPosition.Center);
                args.Compass.IsLeftIndicatorVisible = CanDock(args.DraggedElement, DockPosition.Left);
                args.Compass.IsTopIndicatorVisible = CanDock(args.DraggedElement, DockPosition.Top);
                args.Compass.IsRightIndicatorVisible = CanDock(args.DraggedElement, DockPosition.Right);
                args.Compass.IsBottomIndicatorVisible = CanDock(args.DraggedElement, DockPosition.Bottom);
            }

            args.Canceled = !(CompassNeedsToShow(args.Compass));
        }

        private void OnOpenDialogCommandExecuted(object obj)
        {
            RadWindow.Alert(
                new DialogParameters
                {
                    Content = string.Format("{0}'s command executed.", obj.ToString()),
                    Header = "Telerik"
                });
        }

        private Enums.PaneType GetPaneType(RadPane pane)
        {
            return ConditionalDockingHelper.GetPaneType(pane);
        }

        /// <summary>
        /// Determines if the Docking's Top, Bottom, Left and Right compasses should be shown for the Dragged Pane
        /// </summary>
        private bool CanDock(RadPane paneToDock, DockPosition position)
        {
            var paneToDockType = GetPaneType(paneToDock);
            switch (paneToDockType)
            {
                case Enums.PaneType.Normal:
                    return true;
                case Enums.PaneType.Restricted:
                    return position != DockPosition.Center && position != DockPosition.Top && position != DockPosition.Bottom;
                default:
                    return false;
            }
        }

        private bool CanDock(object dragged, DockPosition position)
        {
            var splitContainer = dragged as RadSplitContainer;
            return !splitContainer.EnumeratePanes().Any(p => !CanDock(p, position));
        }
    }
}